
import django_couch
from django.conf import settings
from django_couch.auth.utils import check_password
from django_couch.auth.models import User


if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl.client import ResourceNotFound
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq.client import ResourceNotFound
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")


class CouchBackend(object):

    supports_object_permissions = False
    supports_anonymous_user = False
    supports_inactive_user = True

    def __init__(self):
        print 'aa'
        self.db = django_couch.db(settings.COUCHDB_AUTH_DB)

    def get_user(self, user_id):
        print 'get_user', user_id
        try:
            user = self.db[user_id]
            if len(self.db.view(settings.COUCHDB_AUTH_VIEW, key=user.username, limit=1).rows):
                return User(user, _db = self.db)
            else:
                return None
        except ResourceNotFound:
            return None


    def authenticate(self, username, password):
        print 'auth user', username
        rows = self.db.view(settings.COUCHDB_AUTH_VIEW, key = username.lower(), include_docs = True, limit = 1).rows

        if len(rows) and check_password(password, rows[0].value):
            return User(rows[0].doc, _db = self.db)
        pass



# -*- coding: utf-8 -*-

import sys
from django.core.management.base import BaseCommand
from django.conf import settings

import django_couch
from couchdbcurl.client import ResourceConflict

class Command(BaseCommand):

    help = u'Creates basic views: types and auth'
    
    def execute(self, db_key, *args, **options):
        
        db = django_couch.db(db_key)
        
        if not args or 'types' in args:
            print "Creating types/list view"
            try:
                db['_design/types'] = {
                    'language': 'javascript',
                    'views': {
                        'list': {
                            'map': "function(doc) {\n  emit(doc.type, 1\t);\n}",
                            'reduce': "function(keys, values) {\n  return sum(values);\n}"
                        }
                    }
                }
                print "  Done"
            except ResourceConflict:
                print "  Can't create: conflict"

        if not args or 'auth' in args:
            print "Creating auth/admins view"
            try:
                db['_design/auth'] = {
                    'language': 'javascript',
                    'views': {
                        'admins': {
                            'map': "function(doc) {\n  if ((doc.type == 'user') && (doc.admin)) {\t\n    emit(doc.username, doc.password);\n  }\t\n}",
                            },
                        "list": {
                            'map': "function(doc) {\n  if ((doc.type == 'user')) {\n    emit(doc.username, doc.password);\n  }\n}"}}
                }
                print "  Done"
            except ResourceConflict:
                print "  Can't create: conflict"

        
        
            



        




import time
import sys
import random
import json

import requests

def main(doc_id):


    i = 0
    time_max = 0
    db_key = random.choice(['smscoin_cluster', 'smscoin_cluster2', 'smscoin_cluster3'])
    print db_key

    #s = requests.Session()
    s = requests
        
    
    while True:
        doc = s.get('http://smscoin:Poh0saece5no@localhost:5984/%s/%s' % (db_key, doc_id)).json()
        doc['position'] += 1
        doc['status'] = 'rev %s reached' % doc['_rev']
        
        start = time.time()

        s.put('http://smscoin:Poh0saece5no@localhost:5984/%s/%s' % (db_key, doc_id), data=json.dumps(doc))
        
        lost = time.time() - start

        if lost > time_max:
            time_max = lost

        if not i % 100:
            print time.time(), i, time_max, doc['_rev']

        i += 1
        
        
    pass


if __name__ == '__main__':
    main(sys.argv[1])

    



from django.conf import settings
from django.core.urlresolvers import reverse

from datetime import datetime
import django_couch

class FlatPageSitemap:
    changefreq = 'never'
    priority = 0.5
    
    def items(self):
        return django_couch.db().view('pages/attachments').rows
    
    def location(self, obj):
        return obj.key[1]
    
    def lastmod(self, obj):
        d = datetime.strptime(obj.key[0], settings.DATETIME_FMT)
        return d.date()

    def images(self, obj):
        return [reverse('attachment', args = [obj.id, name]) for name in obj.value]
        

    def __get(self, name, obj, default=None):
        try:
            attr = getattr(self, name)
        except AttributeError:
            return default
        if callable(attr):
            return attr(obj)
        return attr

    def get_urls(self, **kwargs):
        urls = []
        for item in self.items():
            loc = "%s%s" % (settings.SITE['domain'], self.location(item))
            url_info = {
                'location':   loc,
                'lastmod':    self.__get('lastmod', item, None),
                'changefreq': self.__get('changefreq', item, None),
                'priority':   self.__get('priority', item, None),
                'images':     ['%s%s' % (settings.SITE['domain'], image_url) for image_url in self.images(item)],
            }
            urls.append(url_info)
        return urls





# -*- coding: utf-8 -*-

from pprint import pprint

from django import forms
from annoying.decorators import autostrip

@autostrip
class NodeEditForm(forms.Form):
    hostname = forms.CharField()
    enabled = forms.BooleanField(required=False)
    is_application = forms.BooleanField(required=False)
    is_db = forms.BooleanField(required=False)

    admin_username = forms.CharField(required=False)
    admin_password = forms.CharField(required=False, widget=forms.PasswordInput(render_value=True))

    replica_username = forms.CharField(required=False)
    replica_password = forms.CharField(required=False, widget=forms.PasswordInput(render_value=True))

    def clean_admin_username(self):
        return self.cleaned_data['admin_username'] or None
    
    def clean_admin_password(self):
        return self.cleaned_data['admin_password'] or None
    
    def clean_replica_username(self):
        return self.cleaned_data['replica_username'] or None

    def clean_replica_password(self):
        return self.cleaned_data['replica_password'] or None

@autostrip
class ConfigForm(forms.Form):
    databases = forms.CharField(help_text=u"Comma-separated list of databases (plain names, not config-keys)", required=False)
    admins_names = forms.CharField(help_text=u"Comma-separated list of admin users", required=False)
    admins_roles = forms.CharField(help_text=u"Comma-separated list of admin roles", required=False)
    members_names = forms.CharField(help_text=u"Comma-separated list of member users", required=False)
    members_roles = forms.CharField(help_text=u"Comma-separated list of member roles", required=False)

    def __init__(self, *args, **kwargs):
        if 'initial' in kwargs:
            kwargs['initial']['admins_names'] = ', '.join(kwargs['initial']['security']['admins']['names'])
            kwargs['initial']['admins_roles'] = ', '.join(kwargs['initial']['security']['admins']['roles'])
            kwargs['initial']['members_names'] = ', '.join(kwargs['initial']['security']['members']['names'])
            kwargs['initial']['members_roles'] = ', '.join(kwargs['initial']['security']['members']['roles'])
            kwargs['initial']['databases'] = ', '.join(kwargs['initial']['databases'])

        super(ConfigForm, self).__init__(*args, **kwargs)
    
    def clean_databases(self):
        return sorted([db.strip() for db in self.cleaned_data['databases'].split(',') if db.strip()])

    def clean_admins_names(self):
        return [name.strip() for name in self.cleaned_data['admins_names'].split(',') if name.strip()]
        
    def clean_admins_roles(self):
        return [name.strip() for name in self.cleaned_data['admins_roles'].split(',') if name.strip()]
        
    def clean_members_names(self):
        return [name.strip() for name in self.cleaned_data['members_names'].split(',') if name.strip()]
        
    def clean_members_roles(self):
        return [name.strip() for name in self.cleaned_data['members_roles'].split(',') if name.strip()]
    
    
    def clean(self):
        data = super(ConfigForm, self).clean()
        
        data['security'] = {
            'admins': {
                'names': data.get('admins_names', []),
                'roles': data.get('admins_roles', []),
            },
            'members': {
                'names': data.get('members_names', []),
                'roles': data.get('members_roles', []),
            }
        }
        
        if 'admins_names' in data:
            del(data['admins_names'])
        
        if 'admins_roles' in data:
            del(data['admins_roles'])
        
        if 'members_names' in data:
            del(data['members_names'])
            
        if 'members_roles' in data:
            del(data['members_roles'])
        
        return data
    



from forms import *

doc_types = [
    {
        'name': 'Page',
        'type': 'page',
        'db': None, # means default
        'view': 'pages/list',
        'form': FlatpageForm,
        'attachments': True,
    },

]




from datetime import datetime

from django.conf import settings

if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl.client import Document
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq.client import Document
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")

from django_couch.auth.utils import make_password, check_password
from django.utils.crypto import salted_hmac

class User(Document):

    @property
    def pk(self):
        return self.id

    ## def is_active(self):
    ##     """Fake function. Always returns True"""
    ##     return True

    def is_authenticated(self):
        """Links to self.is_superuser"""
        return '_id' in self

    def get_and_delete_messages(self):
        return None

    def has_perm(self, perm):
        """Permissions system
Django-couch-utils tries to be compliant with basic django permissions system.
So, you must fill user documents with field `permission' which is list of application permission like "module1.permission1".

Example:
{
    username: "...",
    password: "...",
    permissions: ["users.user_edit", "users.user_delete", "files.edit"]
...
}

"""


        return self.is_superuser or perm in self.get_all_permissions()

    def has_perms(self, perm_list):
        """See has_perms() docs"""

        if self.is_superuser:
            return True

        for perm in perm_list:
            if not self.has_perm(perm):
                return False
        return True


    def has_module_perms(self, module):
        """See has_perms() docs"""

        if self.is_superuser:
            return True

        modules = set([perm.split('.')[0] for perm in self.get_all_permissions()])
        return module in modules

    def get_all_permissions(self):

        if not hasattr(self, '_permissions'):
            # cache permissions

            perms = self.get('permissions', [])[:]  # copy perms
            if self.get('groups'):
                rows = self._db.view('_all_docs', keys=self.groups, include_docs=True).rows
                for row in rows:
                    perms.extend(row.doc.permissions)

            self._permissions = set(perms)


        return self._permissions



    def save(self, *args, **kwargs):
        """Save user object and update ``last_login`` field"""

        if hasattr(self, 'backend'):
            backend = self.backend
            del(self.backend)
        else:
            backend = None

        if hasattr(self, 'last_login') and type(self.last_login) == datetime:
            from django.conf import settings
            self.last_login = self.last_login.isoformat()

        if 'update_fields' in kwargs:
            del(kwargs['update_fields'])

        Document.save(self, *args, **kwargs)

        if backend:
            self.backend = backend

    def set_password(self, raw_password):
        self.password = make_password(raw_password)

    def check_password(self, raw_password):
        return check_password(raw_password, self.password)


    @property
    def _meta(self):
        class DjangoFaker():

            @property
            def pk(self):
                return self

            def value_to_string(self, user):
                return user.id


        return DjangoFaker()


    def get_session_auth_hash(self):
        """
        Returns an HMAC of the password field.
        """
        key_salt = "django.contrib.auth.models.AbstractBaseUser.get_session_auth_hash"
        return salted_hmac(key_salt, self.password).hexdigest()



from django import forms
import django_couch
from django_couch.translit import slugify

class CouchAdminForm(forms.Form):
    TYPE = None
    PREFIX = None
    ID_FIELD = None
    
    def __init__(self, *args, **kwargs):
        forms.Form.__init__(self, *args, **kwargs)
        if not self.TYPE or not self.PREFIX or not self.ID_FIELD:
            raise Exception('Please, specify TYPE, PREFIX and ID_FIELD in your form class')
    
    def save(self, files = None):
        self.initial.update(self.cleaned_data)
        
        if not self.initial.get('_id'):
            self.initial.type = self.TYPE
            if isinstance(self.ID_FIELD, list):
                id_field = slugify('-'.join([self.cleaned_data[s] for s in self.ID_FIELD]))
            else:
                id_field = slugify(self.cleaned_data[self.ID_FIELD])
                
            self.initial.create('%s%s' % (self.PREFIX, id_field))
        else:
            self.initial.save()

        if files:
            for f in files:
                att = files[f]
                self.initial.put_attachment(att, att.name, att.content_type)
                



# -*- coding: utf-8 -*-

from datetime import datetime
import urlparse

import django_couch
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import redirect, render
from django.views.decorators.http import require_POST
from django.conf import settings
from django.contrib import messages

from forms import *

CONFIG_ID = 'cluster_config'
    

@user_passes_test(lambda u: u.get("is_superuser"))
def nodes(request):

    nodes = request.db_cluster.view('nodes/list', include_docs=True).rows
    
    return render(request, 'cluster/nodes.html', {
        'nodes': nodes,
    })


@user_passes_test(lambda u: u.get("is_superuser"))
def node_edit(request, node_id=None):

    if node_id:
        node = request.db_cluster[node_id]
        assert node.type == 'node'
    else:
        node = django_couch.Document(_db=request.db_cluster,
                                     type='node',
                                     date_created=datetime.today().strftime(settings.DATETIME_FMT),
                                     enabled=True,
                                     )

    if request.method == "POST":
        form = NodeEditForm(request.POST, initial=node)
        if form.is_valid():
            node.update(form.cleaned_data)

            if node.id:
                node.save()
            else:
                node.create('n_%s' % node.hostname)

            return redirect('cluster:nodes')
    else:
        form = NodeEditForm(initial=node)
    
    
    return render(request, 'cluster/node_edit.html', {
        'node': node,
        'form': form,
    })



@user_passes_test(lambda u: u.get("is_superuser"))
def node(request, node_id):

    config = request.db_cluster[CONFIG_ID]

    def security_check(security, config):
        
        s = dict(security)
        
        return s == config.security
    
    node = request.db_cluster[node_id]
    assert node.type == 'node'
    
    db_info = {
    }
    
    db_nodes = [row.key for row in request.db_cluster.view('nodes/db').rows if row.key != node.hostname]
    
    if node.is_db:
        server = django_couch.Server('http://%s:%s@%s:5984' % (str(node.admin_username), str(node.admin_password), str(node.hostname)))
        
        db_info['databases'] = []
        
        
        
        for db in config.databases:
            try:
                info = server[db].info()
            except django_couch.ResourceNotFound:
                info = None
                
            if info:
                security = server[db]['_security']
                security_correct = security_check(security, config)
            else:
                security = None
                security_correct = None
            
            
            db_info['databases'].append((db, info, security, security_correct))

        db_info['tasks'] = server.tasks()
        
        try:
            db_info['replications'] = server['_replicator'].view('status/list', reduce=False, include_docs=True).rows
        except django_couch.ResourceNotFound:
            db_info['replications_error'] = 'not_found'
            db_info['replications'] = []
        
        
        db_info['permutations'] = []
        for hostname in db_nodes:
            for db in config.databases:
                db_info['permutations'].append({
                    'hostname': hostname,
                    'db': db,
                })

        for perm in db_info['permutations']:
            for replication in db_info['replications']:
                u = urlparse.urlparse(replication.doc.source)
                if u.hostname == perm['hostname'] and u.path.lstrip('/') == perm['db'] and replication.doc.target == perm['db']:
                    perm['status'] = replication.key
        
        

        
    return render(request, 'cluster/node.html', {
        'node': node,
        'db_info': db_info,
    })


@require_POST
@user_passes_test(lambda u: u.get("is_superuser"))
def node_actions(request, node_id):

    node = request.db_cluster[node_id]
    assert node.type == 'node' and node.is_db, "Invalid node id supplied"

    config = request.db_cluster[CONFIG_ID]

    cmd = request.POST['cmd']
    server = django_couch.Server('http://%s:%s@%s:5984' % (str(node.admin_username), str(node.admin_password), str(node.hostname)))
    
    if cmd == 'replication_create':
        
        hostname = request.POST['hostname']
        
        db_nodes = [row.key for row in request.db_cluster.view('nodes/db').rows if row.key != node.hostname]
        assert hostname in db_nodes, "Invalid source hostname supplied"
        
        db = request.POST['db']
        assert db in config.databases, "Invalid DB supplied"
        
        
        
        try:
            source_node = request.db_cluster.view('nodes/db', key=hostname, include_docs=True).rows[0].doc
        except django_couch.ResourceNotFound:
            raise Http404("Source not found")

        source = 'http://%s:%s@%s:5984/%s' % (source_node.replica_username, source_node.replica_password, source_node.hostname, db)

        doc = {
            'source': source,
            'target': db,
            'continuous': True,
            'user_ctx': {"roles": ["_admin"]},
        }
        
        msg = u'Replication started from %s/%s to %s' % (hostname, db, node.hostname)
        messages.info(request, msg)
        
        doc_id = '%s_%s' % (hostname, db)
        
        if doc_id in server['_replicator']:
            del(server['_replicator'][doc_id])
            
        server['_replicator'][doc_id] = doc
        
        
    elif cmd == 'security_fix':
        db = request.POST['db']
        assert db in config.databases, "Invalid DB supplied"

        msg = u'Security doc updated for %s at %s' % (db, node.hostname)
        messages.info(request, msg)
            
        # we need to copy, because it may be modified by couchdbcurl library
        server[db]['_security'] = config.security.copy()

    elif cmd == 'replication_view_create':

        doc = {
            "language": "javascript",
            "views": {
                "list": {
                    "map": "function(doc) {\t\n  emit(doc._replication_state, null);\n}",
                    "reduce": "_count"
                }
            }
        }
        
        doc_id = '_design/status'
        
        server['_replicator'][doc_id] = doc
        
        msg = u'Replications status view created at %s' % (node.hostname)
        messages.info(request, msg)

    elif cmd == 'db_create':
        db = request.POST['db']
        assert db in config.databases, "Invalid DB supplied"

        assert not db in server, "Database already exists"
        
        server.create(db)
        
        msg = u'Database %s created at %s' % (db, node.hostname)
        messages.info(request, msg)
        
        
        
    return redirect('cluster:node', node.id)



@user_passes_test(lambda u: u.get("is_superuser"))
def config(request):
    
    if not CONFIG_ID in request.db_cluster:
        request.db_cluster[CONFIG_ID] = {
            'databases': [],
            'security': {
                'admins': {
                    'names': [],
                    'roles': [],
                },
                'members': {
                    'names': [],
                    'roles': [],
                },
            },
        }
    
    nodes_config = request.db_cluster[CONFIG_ID]
    

    if request.method == "POST":
        form = ConfigForm(request.POST, initial=nodes_config.copy())

        if form.is_valid():
            nodes_config.update(form.cleaned_data)

            pprint(nodes_config)
            nodes_config.save()

            return redirect(request.path)
    else:
        form = ConfigForm(initial=nodes_config.copy())
        
    
    return render(request, 'cluster/config.html', {
        'nodes_config': nodes_config,
        'form': form,
    })




import sys
from django.core.management.base import BaseCommand
from django.conf import settings
from django.contrib.auth.models import User as DjangoUser
from django_couch.auth.models import User

import django_couch

if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl.client import ResourceConflict
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq.client import ResourceConflict
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")


class Command(BaseCommand):

    help = u'Creates admin user'

    def execute(self, db_key, username, *args, **kwargs):

        db = django_couch.db(db_key)
        skel = {
            'type': 'user',
            'is_staff': True,
            'is_superuser': True,
            'username': username,
        }

        u = User(skel, _db = db)
        password = DjangoUser.objects.make_random_password()
        u.set_password(password)
        #password_hash = u.password


        u.create('u_%s' % username)
        print 'User %s created. Document id: %s, password: %s' % (username, u.id, password)

        ## try:
        ##     db['u_%s' % username] = {"is_superuser": True, "is_staff": True, "last_login": None, "password": password_hash, "type": "user", "username": username}
        ##     print "User %s created. Password is: %s" % (username, password)
        ## except ResourceConflict:
        ##     print "Create user failed: conflict"



from django.utils.encoding import smart_str
from django.http import HttpResponse, Http404
from django.template import loader

def sitemap(request, sitemaps):
    urls = []
    for site in sitemaps.values():
        urls.extend(site().get_urls())
    
    xml = smart_str(loader.render_to_string('sitemap.xml', {'urlset': urls}))
    return HttpResponse(xml, mimetype='application/xml')



from django_couch.admin import Admin

from annoying.decorators import render_to
from django.http import Http404, HttpResponse
from django.shortcuts import redirect
from django.contrib.auth.decorators import user_passes_test
from django.core.urlresolvers import reverse
from django.conf import settings

if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl.client import Document
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq.client import Document
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")

def _find_app(app_name):
    app_item = None
    for item in Admin().apps:
        if item['name'] == app_name:
            app_item = item
            break

    if not app_item:
        raise Http404('Undefined application')
    return app_item

def _find_doc_type(app_item, doc_type):
    doc_item = None
    for item in app_item['doc_types']:
        if item['type'] == doc_type:
            doc_item = item
            break

    if not doc_item:
        raise Http404('Undefined document type')

    return doc_item



@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/root.html')
def root(request):
    admin = Admin()
    #print admin.apps

    return {'apps': admin.apps}


@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/app.html')
def app(request, app):

    app_item = _find_app(app)

    return {'app': app_item}


@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/documents.html')
def documents(request, app, doc_type):
    app_item = _find_app(app)
    doc_item = _find_doc_type(app_item, doc_type)

    view = request._db(app_item.get('db')).view(doc_item['view'], descending = doc_item.get('descending', False))

    return {
        'app': app_item,
        'doc_type': doc_type,
        'total': view.total_rows,
        'rows': view.rows,
    }


@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/document.html')
def document(request, app, doc_type, doc_id = None):
    app_item = _find_app(app)
    doc_item = _find_doc_type(app_item, doc_type)
    if doc_id:
        document = request._db(app_item.get('db'))[doc_id]
    else:
        document = Document(_db = request._db(app_item.get('db')))


    form_class = doc_item.get('form')
    form = None

    if form_class:
        form = form_class(initial = document)

        if request.method == 'POST':
            form = form_class(request.POST, request.FILES, initial = document)
            if form.is_valid():
                form.save(files = request.FILES)
                return redirect('couchdb:documents', app, doc_type)


    document.attachments = document.get('_attachments', {})

    return {
        'app': app_item,
        'doc_item': doc_item,
        'doc_type': doc_type,
        'doc_id': doc_id,
        'document': document,
        'form': form,
    }



@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/document_delete.html')
def document_delete(request, app, doc_type, doc_id):
    app_item = _find_app(app)
    doc_item = _find_doc_type(app_item, doc_type)
    document = request._db(app_item.get('db'))[doc_id]

    if request.method == 'POST':
        document.delete()
        return redirect('couchdb:documents', app, doc_type)

    return {
        'app': app_item,
        'doc_type': doc_type,
        'doc_id': doc_id,
    }


@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@user_passes_test(lambda u: getattr(u, 'admin', False))
def attachment(request, app, doc_type, doc_id, filename):
    app_item = _find_app(app)
    doc_item = _find_doc_type(app_item, doc_type)

    try:
        document = request._db(app_item.get('db'))[doc_id]
    except ResourceNotFound:
        raise Http404('Document not found')


    if filename in document.get('_attachments', {}):
        return HttpResponse(document.get_attachment(filename), mimetype = document._attachments[filename]['content_type'])
    else:
        raise Http404('File not found')

@user_passes_test(settings.COUCHDB_AUTH_ADMIN_LAMBDA)
@render_to('admin/attachment_delete.html')
def attachment_delete(request, app, doc_type, doc_id, filename):
    app_item = _find_app(app)
    doc_item = _find_doc_type(app_item, doc_type)

    try:
        document = request._db(app_item.get('db'))[doc_id]
    except ResourceNotFound:
        raise Http404('Document not found')


    if filename in document.get('_attachments', {}):
        if request.method == 'POST':
            document.delete_attachment(filename)
            return redirect('couchdb:document', app, doc_type, document.id)
    else:
        raise Http404('File not found')


    return {
        'app': app_item,
        'doc_item': doc_item,
        'doc_type': doc_type,
        'doc_id': doc_id,
        'document': document,
        'filename': filename,
        'file': document._attachments[filename]
    }



from django.contrib.admin.sites import AdminSite
from django.conf.urls.defaults import patterns, url, include
from django.shortcuts import render_to_response
from django.utils.translation import ugettext_lazy, ugettext as _
from django.utils.text import capfirst
from django.conf import settings
from forms import AdminForm
import os

class CouchdbAdminSite(AdminSite):
    index_template = 'couch_admin/index.html'
    
    def __init__(self):
        self._models = {}
        self.app_name = 'admin'
        self.name = 'admin'

    def index(self, request):
        app_list = self._models.values() #[{'name': self._models[key], 'models': self._models[key]['models']} for key in self._models]
        return render_to_response('couch_admin/index.html', {'app_list': app_list})
    
    def register(self, app, doc, admin_object):
        print 'register:', app, doc, admin_object
        app_url = app
        model_url = '%s/%s/' % (app_url, doc)
        if not app in self._models:
            self._models[app] = {
                'name': app,
                'app_url': app_url,
                'models': [],
                }
            
        self._models[app]['models'].append({
            'admin_url': model_url,
            'doc': doc,
            'name': doc,
            'admin': admin_object,
            'perms': {'change': True, 'add': True},
        })

    def get_urls(self):
        
        
        urlpatterns = patterns('',
            url(r'^$', self.index),
            url(r'^(?P<app>\w+)/$', 'django_couch.admin.app_index'),
            url(r'^(?P<app>\w+)/(?P<model>\w+)/$', 'django_couch.admin.items'),
            url(r'^(?P<app>\w+)/(?P<model>\w+)/(?P<action>add)/$', 'django_couch.admin.item_add'),
            url(r'^(?P<app>\w+)/(?P<model>\w+)/(?P<key>\w+)/delete/$', 'django_couch.admin.item_delete'),
            url(r'^(?P<app>\w+)/(?P<model>\w+)/(?P<key>\w+)/$', 'django_couch.admin.item_edit'),
        )
        ## for app in self._models:
        ##     for model in self._models[app]['models']:
        ##         urlpatterns += patterns('',
        ##                                 url(r'^%s/%s/' % (app, model['doc']), self.app_index) #'django_couch.admin.items')
        ##         )
        return urlpatterns

def app_index(self, app):
    
    return render_to_response('admin/app_index.html',
                              {
                                  'app_list': [site._models[app]],
                                  'title': _('%s administration') % capfirst(app),
                                  'admin_url': '%s/' % app
                                })

def items(self, app, model):
    class Fake2:
        pk = None

    class FakeList(list):
        pass
    
    class Fake1:
        result_count = 10
        full_result_count = 10
        date_hierarchy = None
        lookup_opts = [Fake2(), Fake2()]
        list_display = FakeList()
        formset = None
        result_list = FakeList()
        paginator = None
        page_num = None
        show_all = None
        multi_page = None
        can_show_all = None
        
    m = None
    for i in site._models[app]['models']:
        print i['name']
        if i['name'] == model:
            m = i
            break

    return render_to_response('admin/change_list.html',
                              {
                                  'cl': Fake1(),
                                  'app_list': [site._models[app]],
                                  'has_add_permission': True,
                                  'results': [{'key': 123, 'value': 456}, {'key': 'aaaaa', 'value': 'bbbbbb'}, {'key': 'aaaaa', 'value': 'bbbbbb'}, {'key': 'aaaaa', 'value': 'bbbbbb'}, {'key': 'aaaaa', 'value': 'bbbbbb'}]
                                })
    

def item_edit(request, app, model, key = None, action = None):
    print 'item_edit', app, model, key, action
    class Fake1:
        def get_ordered_objects(self):
            return []

    class Fieldsets(list):
        prepopulated_fields = []
        
    class Fieldset(list):
        name = 'fieldset'
        
    context = {
        'opts': Fake1(),
        'change': None,
        'is_popup': None,
        'save_as': {},
        'has_delete_permission': True,
        'show_delete': True,
        'has_add_permission': True,
        'has_change_permission': True,
        'add': True,
        'adminform': Fieldsets([Fieldset([AdminForm()])]),
    }
    return render_to_response('admin/change_form.html', context)


def item_delete(request, app, model, key):
    print 'item_delete', app, model, key
    pass

    

site = CouchdbAdminSite()

OUR_ROOT = os.path.realpath(os.path.dirname(__file__))
template_dir = os.path.join(OUR_ROOT, 'templates')

settings.TEMPLATE_DIRS = list(settings.TEMPLATE_DIRS) + [template_dir]



import os
from django.conf import settings
from pprint import pprint

MY_ROOT = os.path.realpath(os.path.dirname(__file__))

settings.TEMPLATE_DIRS = list(settings.TEMPLATE_DIRS) + [os.path.join(MY_ROOT, 'templates')]

class Admin(object):
    apps = []



def autodiscover():
    admin = Admin()
    for app in settings.INSTALLED_APPS:
        m = __import__(app, globals(), locals(), ['admin'], -1)

        if hasattr(m, 'admin'):
            doc_types = []
            urls = []
            if hasattr(m.admin, 'doc_types'):
                doc_types = m.admin.doc_types
                
            if hasattr(m.admin, 'urls'):
                urls = m.admin.urls

            if doc_types or urls:
                admin.apps.append({
                    'app': app,
                    'name': app[app.rfind('.') + 1:],
                    'doc_types': doc_types,
                    'urls': urls,
                })
            
            



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from django.core.management.base import BaseCommand
from django.conf import settings

import django_couch
#from optparse import make_option

import multiprocessing

class Pinger(multiprocessing.Process):
    def __init__(self, verbosity, db_key, view_name, func_name):
        multiprocessing.Process.__init__(self)

        self.verbosity = verbosity
        self.db_key = db_key
        self.view_name = view_name
        self.func_name = func_name
        
    def run(self):
        db = django_couch.db(self.db_key)
        if self.verbosity >= 2:
            print 'quering view %s/%s' % (self.view_name, self.func_name)
        db.view('%s/%s' % (self.view_name, self.func_name), limit = 0).rows
                

class Command(BaseCommand):

    help = u'Requests all couchdb views. Usage: ./manage.py couch_ping <db>, where <db> is couchdb definition from settings.py'

    def execute(self, *args, **options):
        
        verbosity = int(options.get('verbosity'))
        
        workers = []
        
        if not len(args):
            print "No database to ping"
            sys.exit(1)
            
        for db_key in args:
            if verbosity >= 2:
                print "Using database", db_key
                print "Settings data:", settings.COUCHDB[db_key]

            db = django_couch.db(db_key)

            for row in django_couch.design_docs(db):
                nop, view = row.id.split('/', 1)

                d = db[row.id]

                for function in d.get('views', []):
                    worker = Pinger(verbosity, db_key, view, function)
                    workers.append(worker)
                    worker.start()

        for worker in workers:
            if verbosity > 1:
                print "Waiting for worker %s (%s/%s @ %s)..." % (worker, worker.view_name, worker.func_name, worker.db_key)
            worker.join()
            if verbosity > 1:
                print '  done'
        
        
        
            



        





import hashlib
import random
from functools import partial

from django.conf import settings

if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl import Server, Document, ResourceConflict, ResourceNotFound
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq import Server, Document, ResourceConflict, ResourceNotFound
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")


from .translit import slugify

def db(name=None, partitioning_data=None):
    """name is a key in settings.COUCHDB. If None, returns default database.
    Examples::

      settings.COUCHDB = {
        'db1': {
          'server': 'http://node1:5984',
          'database': 'database1',
          'default' : True,
        },
        'db2': {
          'server': 'http://node1:5984',
          'database': 'database2',
        },
        'partitioned_db': {
          'server': 'http://node1:5984',
          'database': 'database_%s',
          'partitioning_function': lambda dbname, params: dbname % params.strftime('%Y'),
        },
      }

    Next, you may access django databases as::

      import django_couch
      django_couch.db() # Default database (db1)
      django_couch.db('db1') # Database1 (key - db1) @ http://node1:5984/database1
      django_couch.db('db2') # Database2 (key - db2) @ http://node1:5984/database2

      django_couch.db('partitioned_db')() # Database3 (key - partitioned_db) @ http://node1:5984/partitioned_db_2012
      django_couch.db('partitioned_db')(date(2013, 1, 1)) # Database3 (key - partitioned_db) @ http://node1:5984/partitioned_db_2013


    """

    if not name:
        default = [key for key in settings.COUCHDB if settings.COUCHDB[key].get('default')]
        if len(default) != 1:
            raise Exception("There are no/multiple default database. Please, check settings.COUCHDB value.")
        name = default[0]

    server = Server(settings.COUCHDB[name]['server'])

    row = settings.COUCHDB[name]
    dbname = settings.COUCHDB[name]['database']

    if 'partitioning_function' in row:
        dbname = row['partitioning_function'](dbname, partitioning_data)

    return server[dbname]

## server = Server(settings.COUCHDB_ADDR)

## db = server[settings.COUCHDB_NAME]

class Updater(object):
    """Bulk-updater helper.

    Example usage:

    ::

      updater = Updater(db)
      for document in documents:
          ... # some work here
          updater.append(document)

      updater.push() # last bulk push to ensure all documents are commited to database

    """
    chunk_size = 1000
    """Bulk pushes will be done when ``chunk_size`` documents are in queue"""

    def __init__(self, db):
        self.documents = []
        self.db = db


    def append(self, document):
        """Append document object to bulk queue"""
        self.documents.append(document)

        if len(self.documents) == self.chunk_size:
            return self.push()


    def push(self):
        """Force push documents queue into database"""
        if self.documents:
            res = self.db.update(self.documents)
            self.documents = []
            return res


def all_docs(db, skip_design_docs = False):
    """Iterator-based function, returning all documents from database"""
    for row in view_iterator(db, '_all_docs'):
        if not (skip_design_docs and row.id.startswith('_design/')):
            yield row

def design_docs(db):
    """Iterator-based function, returning design-document from database"""
    return view_iterator(db, '_all_docs', startkey = "_design/", endkey = "_design0", include_docs = True)


def view_iterator(db, view, **kwargs):
    """Iterator helper. You may call this when you need to iterate over a very large number of rows. This function will automatically split requests in 300 result rows, accurate handling ``start_docid``.

    Example::

      for row in view_iterator(db, 'very_large_result_set/will_produce_this_view'):
          ... # some work here


    """

    block_size = kwargs.get('block_size', 300)

    v = db.view(view, limit = block_size, **kwargs)
    while len(v.rows) > 1:
        for row in v.rows[:-1]:
            yield row

        start = v.rows[-1].key
        start_docid = v.rows[-1].id

        if 'startkey' in kwargs:
            del(kwargs['startkey'])

        kwargs['startkey'] = start
        if start_docid:
            kwargs['startkey_docid'] = start_docid

        v = db.view(view, limit = block_size, **kwargs)


    if len(v.rows):
        yield v.rows[0]


def generate_id(db, prefix, suffix_length = 12, id_string = '%s%s'):
    """Generate unique database document ID based on provided prefix and random suffix.
    **Deprecated since generate_doc() function**
    """

    rand = hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]

    key = id_string % (prefix, '')
    max_retry = 1000

    while key in db:
        rand = hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]
        key = id_string % (prefix, '_%s' % rand)
        max_retry -= 1

        if max_retry < 0:
            raise Exception("Retry-limit reached during document id generation")

    return key

def generate_doc(db, prefix, suffix_length = 12, id_string = '%s%s', max_retry = 100, data = {}):
    """Generate doc with unique ID based on provided prefix and random suffix. Retries on duplicate.
    **Deprecated since couchdbcurl.client.Document.create() method.**
    """

    assert not '_id' in data
    assert not '_rev' in data
    assert max_retry > 0
    assert type(max_retry) == int

    #doc = data.copy()

    rand = hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]
    doc_id = id_string % (prefix, '')

    while True:

        try:
            db[doc_id] = data
            break
        except ResourceConflict:
            pass

        max_retry -= 1
        if max_retry < 0:
            raise Exception("Retry-limit reached during document generation")

        rand = hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]
        doc_id = id_string % (prefix, '_%s' % rand)


class CouchMiddleware:
    """Django middleware helper.
    Configuration::

      settings.MIDDLEWARE_CLASSES=(
      ...
      'django_couch.CouchMiddleware',
      )

      settings.COUCHDB = {
        'db1': {
          'server': 'http://node1:5984',
          'database': 'database1',
          'default' : True,
        },
        'db2': {
          'server': 'http://node1:5984',
          'database': 'database2',
        },
      }

    Next, you may access django databases as::

      def some_view(request):
          request.db1 # Database1 (key - db1) @ http://node1:5984/database1
          request.db2 # Database2 (key - db2) @ http://node1:5984/database2

          request._db() # Default database (db1)
          request._db('db1') # Database1
          request._db('db2') # Database2


      """

    def process_request(self, request):
        for dbname in settings.COUCHDB:

            if 'partitioning_function' in settings.COUCHDB[dbname]:
                setattr(request, dbname, partial(db, dbname))
            else:
                setattr(request, dbname, db(dbname))
        request._db = db



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os
import munin

from munin import MuninPlugin


from django.core.management.base import BaseCommand
from django.conf import settings
import django_couch
from time import time

class CouchTypesPlugin(MuninPlugin):
    #title = 
    vlabel = "count"
    category = "couchdb"

    def __init__(self, db_key, *args, **kwargs):
        MuninPlugin.__init__(self, *args, **kwargs)
        self.db = django_couch.db(db_key)
        self.title = "DocTypes grow stats (%s)" % (db_key)

    @property
    def fields(self):
        
        return [(row.key, {
            "label": row.key,
            "info": "%s count" % row.key,
            "type": "COUNTER",
            "min": 0,
            
            }) for row in self.db.view('types/list', reduce = True, group = True).rows]

    def execute(self):
        for row in self.db.view('types/list', reduce = True, group = True).rows:
            print "%s.value %s" % (row.key, row.value)
        

class Command(BaseCommand):
    
    def execute(self, db_key, *args, **options):
        munin = CouchTypesPlugin(db_key)
        sys.argv = [''] + list(args)
        munin.run()
        
    def usage(self, subcommand):
        return 'aaaaaaaa %s' % subcommand





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os
from os.path import join
import urllib
import getopt
from optparse import make_option
from datetime import datetime

from django.core.management.base import BaseCommand
from django.conf import settings

if getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbcurl':
    from couchdbcurl import Server, Document, ResourceNotFound
elif getattr(settings, 'COUCHDB_WRAPPER', None) == 'couchdbrq':
    from couchdbrq import Server, Document, ResourceNotFound
else:
    raise Exception("Couchdb wrapper undefined wrong, please check settings.COUCHDB_WRAPPER")


import django_couch


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--force', '-f', action = 'store_true', dest = 'real_run', default = False, help = u'Real run (change target data)'),
        make_option('--server', '-s', action = 'store', dest = 'server', default = 'http://localhost:5984', help = u'Server address'),
        make_option('--database', '-d', action = 'store', dest = 'database', help = u'Database name'),
        make_option('--path', '-p', action='store', dest='path', help=u'Views root directory, if not defined in settings'),
        make_option('--name', '-n', action = 'store', dest = 'dbkey', help = u'Database key (from settings.py). Will override --server and --database arguments', default = None),
        make_option('--delete', '-r', action = 'store_true', dest = 'delete', help = u'Delete exceed items from database during restore', default = False),

    )

    help = u'Backup and restore couchdb database. Usage: ./manage.py couch [restore|backup]'

    extensions = {
        'javascript' : 'js',
        'python' : 'py'
    }

    extension2lang = dict(zip(extensions.values(), extensions.keys()))


    def execute(self, command = None, **options):
        self.verbose = int(options.get('verbosity')) > 1
        if not command:
            print "No action specified"
            return

        if not command in ['restore', 'backup']:
            print "Unknown command:", command
            return

        if not options.get('database') and not options.get('dbkey'):
            print "No database specified"
            return

        if options.get('dbkey'):
            db = django_couch.db(options['dbkey'])
        else:
            server = Server(options.get('server'))
            db = server[options.get('database')]

        if options.get('path'):
            path = options['path']
            if self.verbose:
                print "Using provided views path"
        elif settings.COUCHDB[options['dbkey']].get('path'):
            path = settings.COUCHDB[options['dbkey']]['path']
            if self.verbose:
                print "Using views path from database definition"
        else:
            path = join(settings.COUCHDB_VIEWS_DIR, options['dbkey'])
            if self.verbose:
                print "Using calculated views path"




        if command == 'backup':
            print "Backing up remote to", path
            self.backup(db, options.get('path'))
        elif command == 'restore':
            print "Restoring local path to remote server", path
            self.restore(db, path, options)


    def backup(self, db, path):

        backup_path = join(path, '_backup', datetime.today().strftime('%Y-%m-%d_%H-%M-%S'))

        os.makedirs(backup_path)

        for item in django_couch.design_docs(db):
            # document subname
            name = os.path.basename(item.id)
            extension = self.extensions[item.doc.get('language', 'javascript')]

            if 'views' in item.doc:
                for view_name in item.doc['views']:
                    dirname = join(backup_path, '%s.%s' % (name, extension), 'views', view_name)
                    os.makedirs(dirname)
                    for view_type in item.doc['views'][view_name]:
                        filename = join(dirname, '%s.%s' % (view_type, extension))
                        print "  Storing %s %s" % (view_type, filename)
                        f = file(filename, 'w')
                        f.write(item.doc['views'][view_name][view_type])
                        f.close()

            if 'filters' in item.doc:
                dirname = join(backup_path, '%s.%s' % (name, extension), 'filters')
                os.makedirs(dirname)
                for filter_name in item.doc['filters']:
                    filename = join(dirname, '%s.%s' % (filter_name, extension))
                    print "  Storing filter %s" % (filename)
                    f = file(filename, 'w')
                    f.write(item.doc['filters'][filter_name])
                    f.close()


    def restore(self, db, path, options):

        real_run = options.get('real_run')
        verbose = int(options.get('verbosity')) > 1

        def macros_load(path):
            macros = {}
            if os.path.exists(join(path, '_macros')):
                for f in os.listdir(join(path, '_macros')):
                    name, ext = os.path.splitext(f)
                    if ext in ['.js', '.py']:
                        if not ext in macros:
                            macros[ext] = {}
                        macros[ext][name] = file(join(path, '_macros', f), 'r').read()

            return macros

        macros = macros_load(path)


        if options['delete']:
            print "Checking for exceed views/filters/shows in design documents"
            delete_docs = []
            for row in django_couch.design_docs(db):
                doc_changed = False
                doc_deleted = False

                doc = db[row.id]
                print os.path.basename(doc.id)
                extension = self.extensions[doc.get('language', 'javascript')]
                doc_name = '%s.%s' % (os.path.basename(doc.id), extension)

                if os.path.exists(join(path, doc_name)):
                    if 'views' in doc:
                        view_path = join(path, doc_name, 'views')
                        if verbose:
                            print '  checking path', view_path

                        if os.path.exists(view_path):
                            for fun_name in doc['views'].keys(): # copy keys for processing, allow to delete them from loop
                                if verbose:
                                    print '    checking function', fun_name

                                if not os.path.exists(join(view_path, fun_name)):
                                    del(doc['views'][fun_name])
                                    if verbose:
                                        print '      function does not exists. Deleting'
                                    doc_changed = True

                            pass
                        else:
                            changed = True
                            del(doc['views'])
                            if verbose:
                                print "  Deleting unexistent views entry"
                    for key in ['filters', 'shows']:
                        if key in doc:
                            key_path = join(path, doc_name, key)
                            if verbose:
                                print '  checking path', key_path

                            if os.path.exists(key_path):
                                for fun_name in doc[key].keys():
                                    if verbose:
                                        print '    checking function', fun_name
                                    if not os.path.exists(join(key_path, '%s.%s' % (fun_name, extension))):
                                        del(doc[key][fun_name])
                                        if verbose:
                                            print '    function does not exists. Deleting'
                                        doc_changed = True
                            else:
                                doc_changed = True
                                del(doc[key])
                                if verbose:
                                    print "  Deleting unexistent %s entry" % key

                else:
                    if verbose:
                        print "  Deleting whole document"
                    doc_deleted = True
                    delete_docs.append(row.id)

                if not doc_deleted and doc_changed:
                    if real_run:
                        doc.save()
                    else:
                        if verbose:
                            print "Fake run. Skipping doc save"

            for doc_id in delete_docs:
                if real_run:
                    doc = db[doc_id]
                    doc.delete()
                else:
                    if verbose:
                        print "Fake run. Skipping doc deleting"




        updated = 0
        created = 0
        skipped = 0

        for item in os.listdir(path):
            if not '.' in item:
                if verbose:
                    print "Ignoring", item
                continue

            doc_name, extension = item.split('.', 1)
            if extension not in self.extension2lang:
                if verbose:
                    print "Ignoring", item
                continue

            print item

            language = self.extension2lang[extension]

            doc_id = '_design/%s' % doc_name
            changed = False

            try:
                doc = db[doc_id]
                creating = False
                if verbose:
                    print "Design-document %s exists" % doc_id
            except ResourceNotFound:
                creating = True
                if verbose:
                    print "Design-document %s not exists. Creating new" % doc_id
                doc = Document({
                    '_id': doc_id,
                    'language': language,
                    }, _db = db)


            for key in os.listdir(join(path, item)):
                if key == 'views':
                    for view_name in os.listdir(join(path, item, key)):

                        if view_name.startswith('.'):
                            if verbose:
                                print "Skipping hidden file %s" % view_name
                            continue

                        print "  %s (views)" % view_name
                        map_filename = join(path, item, key, view_name, 'map.%s' % extension)
                        reduce_filename = join(path, item, key, view_name, 'reduce.%s' % extension)
                        map_func = None
                        reduce_func = None

                        if os.path.exists(map_filename):
                            map_func = file(map_filename, 'r').read().decode('utf-8')

                        if os.path.exists(reduce_filename):
                            reduce_func = file(reduce_filename, 'r').read().decode('utf-8')

                        if map_func:
                            if not 'views' in doc:
                                doc['views'] = {}

                            if not view_name in doc['views']:
                                doc['views'][view_name] = {}

                            if map_func != doc['views'][view_name].get('map'):
                                doc['views'][view_name]['map'] = map_func
                                changed = True
                                if verbose:
                                    print "    map changed. Updating"
                            else:
                                if verbose:
                                    print "    map not changed"

                            if reduce_func and reduce_func != doc['views'][view_name].get('reduce'):
                                doc['views'][view_name]['reduce'] = reduce_func
                                changed = True
                                if verbose:
                                    print "    reduce changed. Updating"
                            else:
                                if verbose:
                                    print "    reduce not changed"

                if key in ['filters', 'shows']:
                    for func_filename in os.listdir(join(path, item, key)):
                        if func_filename.startswith('.'):
                            if verbose:
                                print "Skipping hidden file %s" % func_filename
                            continue

                        if not func_filename.endswith('.' + extension):
                            continue

                        func_name = os.path.splitext(func_filename)[0]
                        print '  %s (%s)' % (func_name, key)
                        if not key in doc:
                            doc[key] = {}

                        filter_func = file(join(path, item, key, func_filename), 'r').read()

                        if filter_func != doc[key].get(func_name):
                            doc[key][func_name] = filter_func
                            changed = True
                            if verbose:
                                print "      differs. Updating"
                        else:
                            if verbose:
                                print "      same"


            if changed:
                if creating:
                    created += 1
                else:
                    updated += 1

                if real_run:
                    print "Changed. Saving..."
                    doc.save()
                else:
                    print "Fake run. Skipping changes"
            else:
                skipped += 1

        if not real_run:
            print "Fake run done"
        print "%d created, %d updated, %d skipped" % (created, updated, skipped)



from annoying.decorators import render_to
from django.shortcuts import render_to_response
import django_couch

from json import loads

def docs_list(request, *args, **kwargs):
    db_key = kwargs.get('db_key')
    template = kwargs.get('template')
    view_args = kwargs.get('view_args', {}).copy()
    view = kwargs.get('view')
    
    db = django_couch.db(db_key)
    
    # some magic here
    if 'startkey' in view_args:
        view_args['startkey'] = loads(view_args['startkey'] % kwargs)
        
    if 'endkey' in view_args:
        view_args['endkey'] = loads(view_args['endkey'] % kwargs)

    
    view_res = db.view(view, **view_args)
    rows = view_res.rows
    
    
    
    return render_to_response(template, {'rows': rows, 'kwargs': kwargs})




import hashlib
import random

def check_password(password, hash):
    raise NotImplemented("This code is not implemented for this django version")

def make_password(password):
    raise NotImplemented("This code is not implemented for this django version")
    

try:
    # for django 1.4
    from django.contrib.auth.hashers import check_password, make_password
except ImportError:
    # for django 1.3

    try:
        from django.contrib.auth.models import check_password
    except ImportError:
        pass
    
    
    def make_password(password, salt=None):
        """Pre-django 1.4 hashing scheme"""

        if not salt:
            salt = hashlib.sha1(str(random.random()) + str(random.random())).hexdigest()[:5]
            
        hsh = hashlib.sha1(salt + password.encode('utf-8')).hexdigest()
        return 'sha1$%s$%s' % (salt, hsh)



# -*- coding: utf-8 -*-

import urlparse
from datetime import datetime

from django import template

register = template.Library()


@register.filter
def hide_url_password(value):
    u = urlparse.urlparse(value)
    if u.hostname:
        # do something only if url is really URL
        if u.username:
            value_format = '%(scheme)s://%(username)s:%(password)s@%(hostname)s%(path)s'
        else:
            value_format = '%(scheme)s://%(hostname)s%(path)s'

        if u.params:
            value_format += '?%(query)s'

        if u.fragment:
            value_format += '#%(fragment)s'


        value = value_format % {
            'scheme': u.scheme,
            'username': u.username,
            'password': '*' * 8,
            'hostname': u.hostname,
            'path': u.path,
            'query': u.query,
            'fragment': u.fragment,
        }
        
    
    return value


@register.filter
def unix(timestamp):
    if isinstance(timestamp, int) or isinstance(timestamp, long) or isinstance(timestamp, float):
        return datetime.fromtimestamp(timestamp)




#!/usr/bin/env python

from setuptools import setup, find_packages
version = '1.6.1'

if __name__ == '__main__':
    setup(name='django-couch-utils',
          version=version,
          description='Handy tools and helpers for django projects, powered by CouchDB',
          author='Alexey Loshkarev',
          author_email='elf2001@gmail.com',
          url='https://bitbucket.org/angry_elf/django-couch-utils/',
          packages=find_packages(),
          #package_data={'django_couch.admin': 'templates'},
          license='GPL',
          classifiers=[
              "Development Status :: 5 - Production/Stable",
              "Intended Audience :: Developers",
              "License :: OSI Approved :: GNU General Public License (GPL)",
              "Natural Language :: English",
              "Programming Language :: Python",
              "Topic :: Software Development :: Libraries :: Python Modules",
              ],
          install_requires=['couchdb-python-curl'],
          package_data={'django_couch.admin': ['templates/admin/*.html'],
                        'django_couch': ['templates/cluster/*.html'],
                        'django_couch.sitemaps': ['templates/*.xml'],
                        },
          include_package_data=True,
          #data_files = ['admin/templates/admin/base.html']
          )



from django_couch.admin.forms import CouchAdminForm
from django import forms

class FlatpageForm(CouchAdminForm):
    TYPE = 'page'
    PREFIX = 'p_'
    ID_FIELD = 'title'
    
    title = forms.CharField(max_length = 255)
    url = forms.CharField(max_length = 255)
    keywords = forms.CharField(max_length = 64000, required = False)
    description = forms.CharField(max_length = 64000, required = False)
    text = forms.CharField(max_length = 256000, required = False, widget = forms.Textarea(attrs = {'cols': 80, 'rows': 20}))
    date = forms.CharField(max_length = 32)
    



from django.conf.urls.defaults import patterns, url

from views import *

urlpatterns = patterns(
    '',
    url(r'^$', nodes, name='nodes'),
    url(r'^config/$', config, name='config'),
    url(r'^-/edit/$', node_edit, name='node_edit'),
    url(r'^([^\/]+)/edit/$', node_edit, name='node_edit'),
    url(r'^([^\/]+)/$', node, name='node'),
    url(r'^([^\/]+)/actions/$', node_actions, name='node_actions'),
)



from django.conf.urls.defaults import *
from django_couch.admin.views import *
from django.conf import settings

_patterns = [
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/(?P<doc_id>.+)/(?P<filename>.+)/attachment/$', attachment, name = 'attachment'),
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/(?P<doc_id>.+)/(?P<filename>.+)/attachment/delete/$', attachment_delete, name = 'attachment_delete'),
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/(?P<doc_id>.+)/delete/$', document_delete, name='document_delete'),
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/add/$', document, name='document_add'),
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/(?P<doc_id>.+)/$', document, name='document'),
    url(r'^crud/(?P<app>.+)/(?P<doc_type>.+)/$', documents, name='documents'),
    url(r'^crud/(?P<app>.+)/$', app, name='app'),
    url(r'^login/$', 'django.contrib.auth.views.login', {'template_name' : 'admin/login.html'}, name='login'),
    url(r'^logout/$', 'django.contrib.auth.views.logout', {'next_page' : '/'}, name='logout'),
    url('^$', root, name='root'),
]

for app in settings.INSTALLED_APPS:
    m = __import__(app, globals(), locals(), ['admin'], -1)
    if hasattr(m, 'admin') and hasattr(m.admin, 'urls'):
        _patterns.extend([i['url'] for i in m.admin.urls])


urlpatterns = patterns(
    '',
    *_patterns
)





from django_couch.admin.forms import CouchAdminForm
from django import forms
from django_couch.auth.models import User

class UserForm(CouchAdminForm):
    TYPE = 'user'
    PREFIX = 'u_'
    ID_FIELD = 'username'
    
    username = forms.CharField(max_length = 64)
    password = forms.CharField(max_length = 128)
    is_staff = forms.BooleanField(required = False)
    admin = forms.BooleanField(required = False)

    def save(self, *args, **kwargs):
        if not self.cleaned_data['password'].startswith('sha1'):
            user = User(self.initial)
            user.set_password(self.cleaned_data['password'])
            self.cleaned_data['password'] = user.password
        
        CouchAdminForm.save(self, *args, **kwargs)




import json

from annoying.decorators import render_to
from django.http import Http404, HttpResponse
from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string

class FlatpageFallbackMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 404:
            return response # No need to check for a flatpage for non-404 responses.
        try:
            return flatpage(request)
        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response
        except:
            if settings.DEBUG:
                raise
            return response

def flatpage(request):

    rows = request.db.view(settings.COUCHDB_FLATPAGES, key=request.path, include_docs=True, limit=1).rows
    if rows:
        row = rows[0]
        if request.is_ajax():
            doc = row.doc.copy()
            doc['text_rendered'] = render_to_string('static_text.html', {'doc': row.doc})
            
            return HttpResponse(json.dumps(doc, ensure_ascii=False, indent=4), mimetype="application/json")
        else:
            return render_to_response(row.doc.get('template', 'static.html'), {'doc': row.doc}, RequestContext(request), mimetype=row.doc.get('content_type', 'text/html'))
    else:
        raise Http404('Page not found')




# -*- coding: utf-8 -*-
# License: GNU GPL2
# Author: Pythy <the.pythy@gmail.com>
"""
Simple transliteration
"""

__id__ = __revision__ = "$Id: translit.py 32 2006-10-28 07:10:43Z the.pythy $"
__url__ = "$URL: https://pythy.googlecode.com/svn/trunk/pytils/pytils/translit.py $"

import re

TRANSTABLE = (
        (u"‘", u"'"),
	(u"’", u"'"),
	(u"«", u'"'),
	(u"»", u'"'),
	(u"–", u"-"),
	(u"…", u"..."),
        (u"№", u"#"),
        ## верхний регистр
        # трехбуквенные замены
        (u"Щ", u"Sch"),
        # при замене русский->английский будет первая замена,
        # т.е. Sch
        # а вот если английский->русский, то вариант SCH и Sch --
        # оба пройдут
        (u"Щ", u"SCH"), 
        # двухбуквенные замены
        (u"Ё", u"Yo"),
        (u"Ё", u"YO"),
        (u"Ж", u"Zh"),
        (u"Ж", u"ZH"),
        (u"Ц", u"Ts"),
        (u"Ц", u"TS"),
        (u"Ч", u"Ch"),
        (u"Ч", u"CH"),
        (u"Ш", u"Sh"),
        (u"Ш", u"SH"),
        (u"Ы", u"Yi"),
        (u"Ы", u"YI"),
        (u"Ю", u"Yu"),
        (u"Ю", u"YU"),
        (u"Я", u"Ya"),
        (u"Я", u"YA"),
        # однобуквенные замены
        (u"А", u"A"),
        (u"Б", u"B"),
        (u"В", u"V"),
        (u"Г", u"G"),
        (u"Д", u"D"),
        (u"Е", u"E"),
        (u"З", u"Z"),
        (u"И", u"I"),
        (u"Й", u"J"),
        (u"К", u"K"),
        (u"Л", u"L"),
        (u"М", u"M"),
        (u"Н", u"N"),
        (u"О", u"O"),
        (u"П", u"P"),
        (u"Р", u"R"),
        (u"С", u"S"),
        (u"Т", u"T"),
        (u"У", u"U"),
        (u"Ф", u"F"),
        (u"Х", u"H"),
        (u"Э", u"E"),
        (u"Ъ", u"`"),
        (u"Ь", u"'"),        
        ## нижний регистр
        # трехбуквенные замены
        (u"щ", u"sch"),
        # двухбуквенные замены
        (u"ё", u"yo"),
        (u"ж", u"zh"),
        (u"ц", u"ts"),
        (u"ч", u"ch"),
        (u"ш", u"sh"),
        (u"ы", u"yi"),
        (u"ю", u"yu"),
        (u"я", u"ya"),
        # однобуквенные замены
        (u"а", u"a"),
        (u"б", u"b"),
        (u"в", u"v"),
        (u"г", u"g"),
        (u"д", u"d"),
        (u"е", u"e"),
        (u"з", u"z"),
        (u"и", u"i"),
        (u"й", u"j"),
        (u"к", u"k"),
        (u"л", u"l"),
        (u"м", u"m"),
        (u"н", u"n"),
        (u"о", u"o"),
        (u"п", u"p"),
        (u"р", u"r"),
        (u"с", u"s"),
        (u"т", u"t"),
        (u"у", u"u"),
        (u"ф", u"f"),
        (u"х", u"h"),
        (u"э", u"e"),
        (u"ъ", u"`"),
        (u"ь", u"'"),
	# для полноты английского алфавита (в slugify)
	# дополняем английскими буквами, которых
	# нет в парах
	(u"c", u"c"),
	(u"q", u"q"),
	(u"y", u"y"),
	(u"x", u"x"),
	(u"C", u"C"),
	(u"Q", u"Q"),
	(u"Y", u"Y"),
	(u"X", u"X"),
	(u"1", u"1"),
	(u"2", u"2"),
	(u"3", u"3"),
	(u"4", u"4"),
	(u"5", u"5"),
	(u"6", u"6"),
	(u"7", u"7"),
	(u"8", u"8"),
	(u"9", u"9"),
	(u"0", u"0"),
        )  #: Translation table

RU_ALPHABET = [x[0] for x in TRANSTABLE] #: Russian alphabet that we can translate
EN_ALPHABET = [x[1] for x in TRANSTABLE] #: English alphabet that we can detransliterate
ALPHABET = RU_ALPHABET + EN_ALPHABET #: Alphabet that we can (de)transliterate


def translify(in_string):
    """
    Translify russian text

    @param in_string: input string
    @type in_string: C{unicode}
    
    @return: transliterated string
    @rtype: C{str}

    @raise TypeError: when in_string is not C{unicode}
    @raise ValueError: when string doesn't transliterate completely
    """
    translit = in_string
    for symb_in, symb_out in TRANSTABLE:
        translit = translit.replace(symb_in, symb_out)

    try:
        translit = str(translit)
    except UnicodeEncodeError:
        raise ValueError("Unicode string doesn't transliterate completely, " + \
                         "is it russian?")

    return translit

def detranslify(in_string):
    """
    Detranslify

    @param in_string: input string
    @type in_string: C{basestring}
    
    @return: detransliterated string
    @rtype: C{str}

    @raise TypeError: when in_string neither C{str}, no C{unicode}
    @raise ValueError: if in_string is C{str}, but it isn't ascii
    """
    # в unicode
    try:
        russian = unicode(in_string)
    except UnicodeDecodeError:
        raise ValueError("We expect ascii when in_string is str type," + \
                         "but now it isn't. Use unicode " + \
                         "in this case.")

    for symb_out, symb_in in TRANSTABLE:
        russian = russian.replace(symb_in, symb_out)

    return russian

def slugify(in_string):
    """
    Prepare string for slug (i.e. URL or file/dir name)

    @param in_string: input string
    @type in_string: C{basestring}

    @return: slug-string
    @rtype: C{str}

    @raise TypeError: when in_string isn't C{unicode} or C{str}
    @raise ValueError: if in_string is C{str}, but it isn't ascii
    """
    try:
        u_in_string = unicode(in_string)
    except UnicodeDecodeError:
        raise ValueError("We expect ascii when in_string is str type," + \
                         "but now it isn't. Use unicode " + \
                         "in this case.")
    


    # convert & to "and"
    u_in_string = re.sub('\&amp\;|\&', ' and ', u_in_string)
    # replace spaces by hyphen
    u_in_string = re.sub('[-\s]+', '-', u_in_string)
    # remove symbols that not in alphabet
    u_in_string = u''.join([symb for symb  in u_in_string if symb in ALPHABET])
    # translify it
    out_string = translify(u_in_string)
    # remove non-alpha
    return re.sub('[^\w\s-]', '', out_string).strip().lower()
        
def dirify(in_string):
    """
    Alias for L{slugify}
    """
    slugify(in_string)

